@@ -36,7 +36,7 @@ from socket import SOCK_DGRAM
 from socket import SOCK_STREAM
 
 import psutil
-from psutil import OSX
+from psutil import MACOS
 from psutil import POSIX
 from psutil import SUNOS
 from psutil import WINDOWS
@@ -54,7 +54,9 @@ else:
 try:
     from unittest import mock  # py3
 except ImportError:
-    import mock  # NOQA - requires "pip install mock"
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        import mock  # NOQA - requires "pip install mock"
 
 if sys.version_info >= (3, 4):
     import enum
@@ -65,14 +67,13 @@ else:
 __all__ = [
     # constants
     'APPVEYOR', 'DEVNULL', 'GLOBAL_TIMEOUT', 'MEMORY_TOLERANCE', 'NO_RETRIES',
-    'PYPY', 'PYTHON', 'ROOT_DIR', 'SCRIPTS_DIR', 'TESTFILE_PREFIX',
+    'PYPY', 'PYTHON_EXE', 'ROOT_DIR', 'SCRIPTS_DIR', 'TESTFILE_PREFIX',
     'TESTFN', 'TESTFN_UNICODE', 'TOX', 'TRAVIS', 'VALID_PROC_STATUSES',
     'VERBOSITY',
     "HAS_CPU_AFFINITY", "HAS_CPU_FREQ", "HAS_ENVIRON", "HAS_PROC_IO_COUNTERS",
     "HAS_IONICE", "HAS_MEMORY_MAPS", "HAS_PROC_CPU_NUM", "HAS_RLIMIT",
-    "HAS_SENSORS_BATTERY", "HAS_BATTERY""HAS_SENSORS_FANS",
+    "HAS_SENSORS_BATTERY", "HAS_BATTERY", "HAS_SENSORS_FANS",
     "HAS_SENSORS_TEMPERATURES", "HAS_MEMORY_FULL_INFO",
-    "HAS_NUM_CTX_SWITCHES",
     # subprocesses
     'pyrun', 'reap_children', 'get_test_subprocess', 'create_zombie_proc',
     'create_proc_children_pair',
@@ -116,6 +117,7 @@ TRAVIS = bool(os.environ.get('TRAVIS'))
 # whether we're running this test suite on Appveyor for Windows
 # (http://www.appveyor.com/)
 APPVEYOR = bool(os.environ.get('APPVEYOR'))
+PYPY = '__pypy__' in sys.builtin_module_names
 
 # --- configurable defaults
 
@@ -124,7 +126,7 @@ NO_RETRIES = 10
 # bytes tolerance for system-wide memory related tests
 MEMORY_TOLERANCE = 500 * 1024  # 500KB
 # the timeout used in functions which have to wait
-GLOBAL_TIMEOUT = 3
+GLOBAL_TIMEOUT = 3 if TRAVIS or APPVEYOR else 0.5
 # test output verbosity
 VERBOSITY = 1 if os.getenv('SILENT') or TOX else 2
 # be more tolerant if we're on travis / appveyor in order to avoid
@@ -136,7 +138,16 @@ if TRAVIS or APPVEYOR:
 # --- files
 
 TESTFILE_PREFIX = '$testfn'
+if os.name == 'java':
+    # Jython disallows @ in module names
+    TESTFILE_PREFIX = '$psutil-test-'
+else:
+    TESTFILE_PREFIX = '@psutil-test-'
 TESTFN = os.path.join(os.path.realpath(os.getcwd()), TESTFILE_PREFIX)
+# Disambiguate TESTFN for parallel testing, while letting it remain a valid
+# module name.
+TESTFN = TESTFN + str(os.getpid())
+
 _TESTFN = TESTFN + '-internal'
 TESTFN_UNICODE = TESTFN + u("-ƒőő")
 ASCII_FS = sys.getfilesystemencoding().lower() in ('ascii', 'us-ascii')
@@ -157,17 +168,43 @@ HAS_PROC_IO_COUNTERS = hasattr(psutil.Process, "io_counters")
 HAS_IONICE = hasattr(psutil.Process, "ionice")
 HAS_MEMORY_FULL_INFO = 'uss' in psutil.Process().memory_full_info()._fields
 HAS_MEMORY_MAPS = hasattr(psutil.Process, "memory_maps")
-HAS_NUM_CTX_SWITCHES = hasattr(psutil.Process, "num_ctx_switches")
 HAS_PROC_CPU_NUM = hasattr(psutil.Process, "cpu_num")
 HAS_RLIMIT = hasattr(psutil.Process, "rlimit")
+HAS_THREADS = hasattr(psutil.Process, "threads")
 HAS_SENSORS_BATTERY = hasattr(psutil, "sensors_battery")
-HAS_BATTERY = HAS_SENSORS_BATTERY and psutil.sensors_battery()
+HAS_BATTERY = HAS_SENSORS_BATTERY and bool(psutil.sensors_battery())
 HAS_SENSORS_FANS = hasattr(psutil, "sensors_fans")
 HAS_SENSORS_TEMPERATURES = hasattr(psutil, "sensors_temperatures")
 
 # --- misc
 
-PYTHON = os.path.realpath(sys.executable)
+
+def _get_py_exe():
+    def attempt(exe):
+        try:
+            subprocess.check_call(
+                [exe, "-V"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except Exception:
+            return None
+        else:
+            return exe
+
+    if MACOS:
+        exe = \
+            attempt(sys.executable) or \
+            attempt(os.path.realpath(sys.executable)) or \
+            attempt(which("python%s.%s" % sys.version_info[:2])) or \
+            attempt(psutil.Process().exe())
+        if not exe:
+            raise ValueError("can't find python exe real abspath")
+        return exe
+    else:
+        exe = os.path.realpath(sys.executable)
+        assert os.path.exists(exe), exe
+        return exe
+
+
+PYTHON_EXE = _get_py_exe()
 DEVNULL = open(os.devnull, 'r+')
 VALID_PROC_STATUSES = [getattr(psutil, x) for x in dir(psutil)
                        if x.startswith('STATUS_')]
@@ -180,10 +217,14 @@ _testfiles_created = set()
 
 
 @atexit.register
-def _cleanup_files():
+def cleanup_test_files():
     DEVNULL.close()
     for name in os.listdir(u('.')):
-        if name.startswith(u(TESTFILE_PREFIX)):
+        if isinstance(name, unicode):
+            prefix = u(TESTFILE_PREFIX)
+        else:
+            prefix = TESTFILE_PREFIX
+        if name.startswith(prefix):
             try:
                 safe_rmpath(name)
             except Exception:
@@ -197,7 +238,7 @@ def _cleanup_files():
 
 # this is executed first
 @atexit.register
-def _cleanup_procs():
+def cleanup_test_procs():
     reap_children(recursive=True)
 
 
@@ -254,7 +295,7 @@ class ThreadTask(threading.Thread):
 # ===================================================================
 
 
-def _cleanup_on_err(fun):
+def _reap_children_on_err(fun):
     @functools.wraps(fun)
     def wrapper(*args, **kwargs):
         try:
@@ -265,7 +306,7 @@ def _cleanup_on_err(fun):
     return wrapper
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def get_test_subprocess(cmd=None, **kwds):
     """Creates a python subprocess which does nothing for 60 secs and
     return it as subprocess.Popen instance.
@@ -287,7 +328,7 @@ def get_test_subprocess(cmd=None, **kwds):
         pyline = "from time import sleep;" \
                  "open(r'%s', 'w').close();" \
                  "sleep(60);" % _TESTFN
-        cmd = [PYTHON, "-c", pyline]
+        cmd = [PYTHON_EXE, "-c", pyline]
         sproc = subprocess.Popen(cmd, **kwds)
         _subprocesses_started.add(sproc)
         wait_for_file(_TESTFN, delete=True, empty=True)
@@ -298,7 +339,7 @@ def get_test_subprocess(cmd=None, **kwds):
     return sproc
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def create_proc_children_pair():
     """Create a subprocess which creates another one as in:
     A (us) -> B (child) -> C (grandchild).
@@ -309,15 +350,14 @@ def create_proc_children_pair():
     _TESTFN2 = os.path.basename(_TESTFN) + '2'  # need to be relative
     s = textwrap.dedent("""\
         import subprocess, os, sys, time
-        PYTHON = os.path.realpath(sys.executable)
         s = "import os, time;"
         s += "f = open('%s', 'w');"
         s += "f.write(str(os.getpid()));"
         s += "f.close();"
         s += "time.sleep(60);"
-        subprocess.Popen([PYTHON, '-c', s])
+        subprocess.Popen(['%s', '-c', s])
         time.sleep(60)
-        """ % _TESTFN2)
+        """ % (_TESTFN2, PYTHON_EXE))
     # On Windows if we create a subprocess with CREATE_NO_WINDOW flag
     # set (which is the default) a "conhost.exe" extra process will be
     # spawned as a child. We don't want that.
@@ -327,7 +367,7 @@ def create_proc_children_pair():
         subp = pyrun(s)
     child1 = psutil.Process(subp.pid)
     data = wait_for_file(_TESTFN2, delete=False, empty=False)
-    os.remove(_TESTFN2)
+    safe_rmpath(_TESTFN2)
     child2_pid = int(data)
     _pids_started.add(child2_pid)
     child2 = psutil.Process(child2_pid)
@@ -337,7 +377,7 @@ def create_proc_children_pair():
 def create_zombie_proc():
     """Create a zombie process and return its PID."""
     assert psutil.POSIX
-    unix_file = tempfile.mktemp(prefix=TESTFILE_PREFIX) if OSX else TESTFN
+    unix_file = tempfile.mktemp(prefix=TESTFILE_PREFIX) if MACOS else TESTFN
     src = textwrap.dedent("""\
         import os, sys, time, socket, contextlib
         child_pid = os.fork()
@@ -371,7 +411,7 @@ def create_zombie_proc():
             conn.close()
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def pyrun(src, **kwds):
     """Run python 'src' code string in a separate interpreter.
     Returns a subprocess.Popen instance.
@@ -383,12 +423,12 @@ def pyrun(src, **kwds):
         _testfiles_created.add(f.name)
         f.write(src)
         f.flush()
-        subp = get_test_subprocess([PYTHON, f.name], **kwds)
+        subp = get_test_subprocess([PYTHON_EXE, f.name], **kwds)
         wait_for_pid(subp.pid)
     return subp
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def sh(cmd, **kwds):
     """run cmd in a subprocess and return its output.
     raises RuntimeError on error.
@@ -452,7 +492,9 @@ def reap_children(recursive=False):
         try:
             subp.terminate()
         except OSError as err:
-            if err.errno != errno.ESRCH:
+            if WINDOWS and err.errno == 6:  # "invalid handle"
+                pass
+            elif err.errno != errno.ESRCH:
                 raise
         if subp.stdout:
             subp.stdout.close()
@@ -631,7 +673,7 @@ def wait_for_file(fname, delete=True, empty=False):
     if not empty:
         assert data
     if delete:
-        os.remove(fname)
+        safe_rmpath(fname)
     return data
 
 
@@ -653,12 +695,34 @@ def call_until(fun, expr):
 
 def safe_rmpath(path):
     "Convenience function for removing temporary test files or dirs"
+    def retry_fun(fun):
+        # On Windows it could happen that the file or directory has
+        # open handles or references preventing the delete operation
+        # to succeed immediately, so we retry for a while. See:
+        # https://bugs.python.org/issue33240
+        stop_at = time.time() + 1
+        while time.time() < stop_at:
+            try:
+                return fun()
+            except WindowsError as _:
+                err = _
+                if err.errno != errno.ENOENT:
+                    raise
+                else:
+                    warn("ignoring %s" % (str(err)))
+                    time.sleep(0.01)
+        raise err
+
     try:
         st = os.stat(path)
         if stat.S_ISDIR(st.st_mode):
-            os.rmdir(path)
+            fun = functools.partial(shutil.rmtree, path)
         else:
-            os.remove(path)
+            fun = functools.partial(os.remove, path)
+        if POSIX:
+            fun()
+        else:
+            retry_fun(fun)
     except OSError as err:
         if err.errno != errno.ENOENT:
             raise
@@ -690,7 +754,7 @@ def create_exe(outpath, c_code=None):
     if c_code:
         if not which("gcc"):
             raise ValueError("gcc is not installed")
-        if c_code is None:
+        if isinstance(c_code, bool):        # c_code is True
             c_code = textwrap.dedent(
                 """
                 #include <unistd.h>
@@ -699,6 +763,7 @@ def create_exe(outpath, c_code=None):
                     return 1;
                 }
                 """)
+        assert isinstance(c_code, str), c_code
         with tempfile.NamedTemporaryFile(
                 suffix='.c', delete=False, mode='wt') as f:
             f.write(c_code)
@@ -708,7 +773,7 @@ def create_exe(outpath, c_code=None):
             safe_rmpath(f.name)
     else:
         # copy python executable
-        shutil.copyfile(sys.executable, outpath)
+        shutil.copyfile(PYTHON_EXE, outpath)
         if POSIX:
             st = os.stat(outpath)
             os.chmod(outpath, st.st_mode | stat.S_IEXEC)
@@ -743,16 +808,21 @@ unittest.TestCase = TestCase
 
 
 def _setup_tests():
-    assert 'PSUTIL_TESTING' in os.environ
-    assert psutil._psplatform.cext.py_psutil_testing()
+    if 'PSUTIL_TESTING' not in os.environ:
+        # This won't work on Windows but set_testing() below will do it.
+        os.environ['PSUTIL_TESTING'] = '1'
+    psutil._psplatform.cext.set_testing()
 
 
 def get_suite():
-    testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
-                   if x.endswith('.py') and x.startswith('test_') and not
-                   x.startswith('test_memory_leaks')]
+    testmods = [os.path.splitext(x)[0] for x in os.listdir(HERE)
+                if x.endswith('.py') and x.startswith('test_') and not
+                x.startswith('test_memory_leaks')]
+    if "WHEELHOUSE_UPLOADER_USERNAME" in os.environ:
+        testmods = [x for x in testmods if not x.endswith((
+                    "osx", "posix", "linux"))]
     suite = unittest.TestSuite()
-    for tm in testmodules:
+    for tm in testmods:
         # ...so that the full test paths are printed on screen
         tm = "psutil.tests.%s" % tm
         suite.addTest(unittest.defaultTestLoader.loadTestsFromName(tm))
@@ -1115,11 +1185,12 @@ if POSIX:
         by this process, copies it in another location and loads it
         in memory via ctypes. Return the new absolutized path.
         """
+        exe = 'pypy' if PYPY else 'python'
         ext = ".so"
         dst = tempfile.mktemp(prefix=dst_prefix, suffix=ext)
         libs = [x.path for x in psutil.Process().memory_maps() if
                 os.path.splitext(x.path)[1] == ext and
-                'python' in x.path.lower()]
+                exe in x.path.lower()]
         src = random.choice(libs)
         shutil.copyfile(src, dst)
         try:
